#!/usr/bin/env python
# coding: utf-8

# In[1]:


import spectools_niw as niw
import numpy as np
import xarray as xr
import warnings
warnings.filterwarnings('ignore')
from importlib import reload
import matplotlib.pyplot as plt
from scipy import stats
import pandas as pd
from matplotlib.colors import BoundaryNorm
cls = plt.rcParams['axes.prop_cycle'].by_key()['color']
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
cm = 1/2.54
def sum_regions(dat):
    gl = np.sum(dat[:,:] * ga01[:,:]).values
    sh = np.sum(dat[0:900,:] * ga01[0:900,:]).values
    nh = np.sum(dat[900:,:] * ga01[900:,:]).values
    eq = np.sum(dat[800:1000,:] * ga01[800:1000,:]).values
    
    return np.round(gl*1e-12,3),np.round(sh*1e-12,3),np.round(nh*1e-12,3),np.round(eq*1e-12,3)

figdir="/home/mpim/m300522/projects/niw_dywin/figs/"


# In[10]:


#dirs
sumdir = "/work/mh0256/m300522/dywin/for_paper/sep2020/"
windir = "/work/mh0256/m300522/dywin/for_paper/feb2021/"

#dicts
flux_allf = {}; flux_inIn = {}; flux_inOlbers = {}
input_allf = {}; input_inIn = {}
mld = {}


#metas
ga01 = xr.open_dataset(sumdir+"../ga_r01.nc").cell_area
mld["win"] = xr.open_dataset(windir+"p_hydro_atMLB_MAXcap700_202101-202102.nc").mld
mld["sum"] = xr.open_dataset(sumdir+"p_hydro_atMLB_MAXcap700ydro_r01_202008-202009.nc").mld



# load data from disk
flux_inIn["win"] = xr.open_dataset(windir+"flux_inIn_sw02_r01_202101-202102.nc").flux_inIn
flux_inIn["sum"] = xr.open_dataset(sumdir+"flux_inIn_sw02_r01_202008-202009.nc").flux_inIn

flux_inOlbers["win"] = xr.open_dataset(windir+"flux_inOlbers_sw4_r01_202101-202102.nc").flux_inIn
flux_inOlbers["sum"] = xr.open_dataset(sumdir+"flux_inOlbers_sw4_r01_202008-202009.nc").flux_inIn

input_inIn["win"] = xr.open_dataset(windir+"input_inIn_sw02_r01_202101-202102.nc").input_inIn 
input_inIn["sum"] = xr.open_dataset(sumdir+"input_inIn_sw02_r01_202008-202009.nc").input_inIn


# In[11]:


# Compute zonal averages
zm_ratio_win = pd.Series(-np.nansum(flux_inOlbers["win"],axis=1) / np.nansum(input_inIn["win"],axis=1)).copy()
zm_ratio_win = zm_ratio_win.rolling(window=30,center=True).mean()

zm_ratio_sum = pd.Series(-np.nansum(flux_inOlbers["sum"],axis=1) / np.nansum(input_inIn["sum"],axis=1)).copy()
zm_ratio_sum = zm_ratio_sum.rolling(window=30,center=True).mean()
#mask out region around equator where values diverge
zm_ratio_win[850:950] = np.nan
zm_ratio_sum[850:950] = np.nan

# apply some smoothing of the zonal averages
zm_flux_win = -pd.Series(np.nansum(flux_inOlbers["win"],axis=1)).rolling(window=30,center=True).mean()
zm_input_win = pd.Series(np.nansum(input_inIn["win"],axis=1)).rolling(window=30,center=True).mean()
zm_flux_sum = -pd.Series(np.nansum(flux_inOlbers["sum"],axis=1)).rolling(window=30,center=True).mean()
zm_input_sum = pd.Series(np.nansum(input_inIn["sum"],axis=1)).rolling(window=30,center=True).mean()


# In[12]:


## SURFACE INPUT WINTER
fig = plt.figure(figsize=(12,9),dpi=100)
xx,yy = np.meshgrid(flux_inIn["win"].lon,flux_inIn["win"].lat)
fs = 18

ax = fig.add_subplot(111)
high=1e-2
cmap = plt.get_cmap('BrBG_r')
norm = BoundaryNorm(np.linspace(-high,high,50), ncolors=cmap.N, clip=False)
axp = plt.pcolormesh(xx,yy,input_inIn["win"],vmax=high,vmin=-high,norm=norm,cmap="BrBG_r")

plt.xlabel("Longitude [°]",fontsize=fs); plt.ylabel("Latitude [°]",fontsize=fs)
ax.set_xticks([60,120,180,240,300]); ax.set_xticklabels([60,120,180,240,300],fontsize=fs);
ax.set_yticks([-60,-30,0,30,60]); ax.set_yticklabels([-60,-30,0,30,60],fontsize=fs);

cbaxes = fig.add_axes([0.92, 0.14, 0.02, 0.55])  
cb = plt.colorbar(axp, cax = cbaxes,ticks = [-8e-3,-4e-3,0,4e-3,8e-3],extend="max")
cb.ax.set_yticklabels(["-8e-3","-4e-3","0","4e-3","8e-3"],fontsize=fs-2)
cb.set_label("NI \n surface \n energy \n flux \n [Wm$^{-2}$]", labelpad=-40, y=1.32,rotation=0,fontsize = fs)
ax.set_ylim(-75,75)

plt.savefig(figdir+"winter_input_single.png",bbox_inches="tight")


# In[13]:


## MLB FLUX WINTER
fig = plt.figure(figsize=(12,9),dpi=100)
xx,yy = np.meshgrid(flux_inIn["win"].lon,flux_inIn["win"].lat)
fs = 18

ax = fig.add_subplot(111)
high=5e-3
cmap = plt.get_cmap('RdBu_r')
norm = BoundaryNorm(np.linspace(-high,high,50), ncolors=cmap.N, clip=False)
axp = plt.pcolormesh(xx,yy,-flux_inOlbers["win"],vmax=high,vmin=-high,norm=norm,cmap="RdBu_r")

plt.xlabel("Longitude [°]",fontsize=fs); plt.ylabel("Latitude [°]",fontsize=fs)
ax.set_xticks([60,120,180,240,300]); ax.set_xticklabels([60,120,180,240,300],fontsize=fs);
ax.set_yticks([-60,-30,0,30,60]); ax.set_yticklabels([-60,-30,0,30,60],fontsize=fs);

cbaxes = fig.add_axes([0.92, 0.14, 0.02, 0.55])  
cb = plt.colorbar(axp, cax = cbaxes,ticks = [-4e-3,-2e-3,0,2e-3,4e-3],extend="max")
cb.ax.set_yticklabels(["-4e-3","-2e-3","0","2e-3","4e-3"],fontsize=fs-2)
cb.set_label("NIW \n energy \n flux \n at MLB \n [Wm$^{-2}$]", labelpad=-40, y=1.32,rotation=0,fontsize = fs)
ax.set_ylim(-75,75)

plt.savefig(figdir+"winter_flux_single.png",bbox_inches="tight")


# In[14]:


## RATIO
fig = plt.figure(figsize=(12,9),dpi=100)
xx,yy = np.meshgrid(flux_inIn["win"].lon,flux_inIn["win"].lat)
fs = 18

ax = fig.add_subplot(111)
cmap = plt.get_cmap('coolwarm')
norm = BoundaryNorm(np.linspace(-1,1,20), ncolors=cmap.N, clip=True)
ratio = -flux_inOlbers["win"]/ input_inIn["win"]
ratio.values[np.abs(flux_inOlbers["win"].values) <1e-4] = 0
axp = plt.pcolormesh(xx,yy,ratio,norm=norm,cmap="PiYG_r")

plt.xlabel("Longitude [°]",fontsize=fs); plt.ylabel("Latitude [°]",fontsize=fs)
ax.set_xticks([60,120,180,240,300]); ax.set_xticklabels([60,120,180,240,300],fontsize=fs);
ax.set_yticks([-60,-30,0,30,60]); ax.set_yticklabels([-60,-30,0,30,60],fontsize=fs);

cbaxes = fig.add_axes([0.92, 0.14, 0.02, 0.55])  
cb = plt.colorbar(axp, cax = cbaxes,ticks = [-1,0.5,0,0.5,1],extend="both")
cb.ax.set_yticklabels([-1,0.5,0,0.5,1],fontsize=fs-2)
cb.set_label("Ratio \n MLB \n flux to \n surface \n flux", labelpad=-30, y=1.32,rotation=0,fontsize = fs)
ax.set_ylim(-75,75)
plt.savefig(figdir+"winter_ratio_single.png",bbox_inches="tight")


# In[30]:


fig = plt.figure(figsize=(12,8),dpi=100)
ls = 3
ax21 = fig.add_subplot(131)
p0, = ax21.plot(zm_ratio_win,flux_inOlbers["win"].lat,lw=lw,color="black")
ax21.set_xlim(0,0.8); ax21.set_ylim(-75,75)
ax21.set_ylabel("Latitude / deg", fontsize=fs)
ax21.set_xlabel("Ratio MLB flux /  surface flux", fontsize=fs)
ax21.set_yticks([-60,-30,0,30,60]); ax21.set_yticklabels([-60,-30,0,30,60],fontsize=fs);
ax21.tick_params(axis='x', colors=p0.get_color(),labelsize=fs)
ax21.set_xticks([0.1,0.3,0.5,0.7,0.9]); ax21.set_xticklabels([0.1,0.3,0.5,0.7,0.9])



ax22 = fig.add_subplot(132)
p1, = ax22.plot(zm_input_win,flux_inOlbers["win"].lat,lw=lw,color=cls[1],label="Surface flux")
p2, = ax22.plot(zm_flux_win,flux_inOlbers["win"].lat,lw=lw,color=cls[0], label ="MLB flux")
ax22.set_xlim(0,4.5); ax22.set_ylim(-75,75)
ax22.set_ylabel("Latitude / deg", fontsize=fs)
ax22.set_xlabel("Energy flux [W]", fontsize=fs)
ax22.set_yticks([-60,-30,0,30,60]); ax22.set_yticklabels([-60,-30,0,30,60],fontsize=fs);
ax22.tick_params(axis='x',labelsize=fs)
ax22.set_xticks([1,3,5,7]); ax22.set_xticklabels([1,3,5,7])
plt.legend(fontsize=fs-2)
plt.tight_layout()
plt.savefig(figdir+"winter_zonal_means_single.png",bbox_inches="tight")


# # SUMMER

# In[17]:


## SURFACE INPUT SUMMER
fig = plt.figure(figsize=(12,9),dpi=100)
xx,yy = np.meshgrid(flux_inIn["sum"].lon,flux_inIn["sum"].lat)
fs = 18

ax = fig.add_subplot(111)
high=1e-2
cmap = plt.get_cmap('BrBG_r')
norm = BoundaryNorm(np.linspace(-high,high,50), ncolors=cmap.N, clip=False)
axp = plt.pcolormesh(xx,yy,input_inIn["sum"],vmax=high,vmin=-high,norm=norm,cmap="BrBG_r")

plt.xlabel("Longitude [°]",fontsize=fs); plt.ylabel("Latitude [°]",fontsize=fs)
ax.set_xticks([60,120,180,240,300]); ax.set_xticklabels([60,120,180,240,300],fontsize=fs);
ax.set_yticks([-60,-30,0,30,60]); ax.set_yticklabels([-60,-30,0,30,60],fontsize=fs);

cbaxes = fig.add_axes([0.92, 0.14, 0.02, 0.55])  
cb = plt.colorbar(axp, cax = cbaxes,ticks = [-8e-3,-4e-3,0,4e-3,8e-3],extend="max")
cb.ax.set_yticklabels(["-8e-3","-4e-3","0","4e-3","8e-3"],fontsize=fs-2)
cb.set_label("NI \n surface \n energy \n flux \n [Wm$^{-2}$]", labelpad=-40, y=1.32,rotation=0,fontsize = fs)
ax.set_ylim(-75,75)

plt.savefig(figdir+"summer_input_single.png",bbox_inches="tight")


# In[18]:


## MLB FLUX SUMMER
fig = plt.figure(figsize=(12,9),dpi=100)
xx,yy = np.meshgrid(flux_inIn["sum"].lon,flux_inIn["sum"].lat)
fs = 18

ax = fig.add_subplot(111)
high=5e-3
cmap = plt.get_cmap('RdBu_r')
norm = BoundaryNorm(np.linspace(-high,high,50), ncolors=cmap.N, clip=False)
axp = plt.pcolormesh(xx,yy,-flux_inOlbers["sum"],vmax=high,vmin=-high,norm=norm,cmap="RdBu_r")

plt.xlabel("Longitude [°]",fontsize=fs); plt.ylabel("Latitude [°]",fontsize=fs)
ax.set_xticks([60,120,180,240,300]); ax.set_xticklabels([60,120,180,240,300],fontsize=fs);
ax.set_yticks([-60,-30,0,30,60]); ax.set_yticklabels([-60,-30,0,30,60],fontsize=fs);

cbaxes = fig.add_axes([0.92, 0.14, 0.02, 0.55])  
cb = plt.colorbar(axp, cax = cbaxes,ticks = [-4e-3,-2e-3,0,2e-3,4e-3],extend="max")
cb.ax.set_yticklabels(["-4e-3","-2e-3","0","2e-3","4e-3"],fontsize=fs-2)
cb.set_label("NIW \n energy \n flux \n at MLB \n [Wm$^{-2}$]", labelpad=-40, y=1.32,rotation=0,fontsize = fs)
ax.set_ylim(-75,75)

plt.savefig(figdir+"summer_flux_single.png",bbox_inches="tight")


# In[19]:


## RATIO
fig = plt.figure(figsize=(12,9),dpi=100)
xx,yy = np.meshgrid(flux_inIn["sum"].lon,flux_inIn["sum"].lat)
fs = 18

ax = fig.add_subplot(111)
cmap = plt.get_cmap('coolwarm')
norm = BoundaryNorm(np.linspace(-1,1,20), ncolors=cmap.N, clip=True)
ratio = -flux_inOlbers["sum"]/ input_inIn["sum"]
ratio.values[np.abs(flux_inOlbers["sum"].values) <1e-4] = 0
axp = plt.pcolormesh(xx,yy,ratio,norm=norm,cmap="PiYG_r")

plt.xlabel("Longitude [°]",fontsize=fs); plt.ylabel("Latitude [°]",fontsize=fs)
ax.set_xticks([60,120,180,240,300]); ax.set_xticklabels([60,120,180,240,300],fontsize=fs);
ax.set_yticks([-60,-30,0,30,60]); ax.set_yticklabels([-60,-30,0,30,60],fontsize=fs);

cbaxes = fig.add_axes([0.92, 0.14, 0.02, 0.55])  
cb = plt.colorbar(axp, cax = cbaxes,ticks = [-1,0.5,0,0.5,1],extend="both")
cb.ax.set_yticklabels([-1,0.5,0,0.5,1],fontsize=fs-2)
cb.set_label("Ratio \n MLB \n flux to \n surface \n flux", labelpad=-30, y=1.32,rotation=0,fontsize = fs)
ax.set_ylim(-75,75)
plt.savefig(figdir+"summer_ratio_single.png",bbox_inches="tight")


# In[39]:


fig = plt.figure(figsize=(12,8),dpi=100)
ax21 = fig.add_subplot(131)
p0, = ax21.plot(zm_ratio_sum,flux_inOlbers["sum"].lat,lw=lw,color="black")
ax21.set_xlim(0,1); ax21.set_ylim(-75,75)
ax21.set_ylabel("Latitude / deg", fontsize=fs)
ax21.set_xlabel("Ratio MLB flux /  surface flux", fontsize=fs)
ax21.set_yticks([-60,-30,0,30,60]); ax21.set_yticklabels([-60,-30,0,30,60],fontsize=fs);
ax21.tick_params(axis='x', colors=p0.get_color(),labelsize=fs)
ax21.set_xticks([0.1,0.3,0.5,0.7,0.9]); ax21.set_xticklabels([0.1,0.3,0.5,0.7,0.9])


ax22 = fig.add_subplot(132)
p1, = ax22.plot(zm_input_sum,flux_inOlbers["sum"].lat,lw=lw,color=cls[1],label="Surface flux")
p2, = ax22.plot(zm_flux_sum,flux_inOlbers["sum"].lat,lw=lw,color=cls[0], label ="MLB flux")
ax22.set_xlim(0,7); ax22.set_ylim(-75,75)
ax22.set_ylabel("Latitude / deg", fontsize=fs)
ax22.set_xlabel("Energy flux [W]", fontsize=fs)
ax22.set_yticks([-60,-30,0,30,60]); ax22.set_yticklabels([-60,-30,0,30,60],fontsize=fs);
ax22.tick_params(axis='x',labelsize=fs)
ax22.set_xticks([1,3,5,7]); ax22.set_xticklabels([1,3,5,7])
plt.legend(fontsize=fs-2)
plt.tight_layout()
plt.savefig(figdir+"summer_zonal_means_single.png",bbox_inches="tight")


# ## Print some numbers

# In[21]:


print("0.2*f around f")
print()

s_flux_win = sum_regions(-1*flux_inIn["win"])
s_flux_sum = sum_regions(-1*flux_inIn["sum"])

s_input_win = sum_regions(input_inIn["win"])
s_input_sum = sum_regions(input_inIn["sum"])

print("SUMMER:", " Input"  ," / ", "MLB Flux  "," / ", "Ratio")
print('Global: ',s_input_sum[0]," / ",s_flux_sum[0],"[TW]"," / ", 
      np.round(s_flux_sum[0]/ s_input_sum[0]*100),"%" )
print('SH:     ',s_input_sum[1]," / ",s_flux_sum[1],"[TW]"," / ", 
      np.round(s_flux_sum[1]/ s_input_sum[1]*100),"%" )
print('NH:     ',s_input_sum[2]," / ",s_flux_sum[2],"[TW]"," / ", 
      np.round(s_flux_sum[2]/ s_input_sum[2]*100),"%" )
print('Equator:',s_input_sum[3]," / ",s_flux_sum[3],"[TW]"," / ", 
      np.round(s_flux_sum[3]/ s_input_sum[3]*100),"%" )
print()
print("WINTER:", " Input"  ," / ", "MLB Flux  "," / ", "Ratio")
print('Global: ',s_input_win[0]," / ",s_flux_win[0],"[TW]"," / ", 
      np.round(s_flux_win[0]/ s_input_win[0]*100),"%" )
print('SH:     ',s_input_win[1]," / ",s_flux_win[1],"[TW]"," / ", 
      np.round(s_flux_win[1]/ s_input_win[1]*100),"%" )
print('NH:     ',s_input_win[2]," / ",s_flux_win[2],"[TW]"," / ", 
      np.round(s_flux_win[2]/ s_input_win[2]*100),"%" )
print('Equator:',s_input_win[3]," / ",s_flux_win[3],"[TW]"," / ", 
      np.round(s_flux_win[3]/ s_input_win[3]*100),"%" )


# In[22]:


print("Olbers filter, only higher than f")
print()

s_flux_win = sum_regions(-1*flux_inOlbers["win"])
s_flux_sum = sum_regions(-1*flux_inOlbers["sum"])

s_input_win = sum_regions(input_inIn["win"])
s_input_sum = sum_regions(input_inIn["sum"])

print("SUMMER:", " Input"  ," / ", "MLB Flux  "," / ", "Ratio")
print('Global: ',s_input_sum[0]," / ",s_flux_sum[0],"[TW]"," / ", 
      np.round(s_flux_sum[0]/ s_input_sum[0]*100),"%" )
print('SH:     ',s_input_sum[1]," / ",s_flux_sum[1],"[TW]"," / ", 
      np.round(s_flux_sum[1]/ s_input_sum[1]*100),"%" )
print('NH:     ',s_input_sum[2]," / ",s_flux_sum[2],"[TW]"," / ", 
      np.round(s_flux_sum[2]/ s_input_sum[2]*100),"%" )
print('Equator:',s_input_sum[3]," / ",s_flux_sum[3],"[TW]"," / ", 
      np.round(s_flux_sum[3]/ s_input_sum[3]*100),"%" )
print()
print("WINTER:", " Input"  ," / ", "MLB Flux  "," / ", "Ratio")
print('Global: ',s_input_win[0]," / ",s_flux_win[0],"[TW]"," / ", 
      np.round(s_flux_win[0]/ s_input_win[0]*100),"%" )
print('SH:     ',s_input_win[1]," / ",s_flux_win[1],"[TW]"," / ", 
      np.round(s_flux_win[1]/ s_input_win[1]*100),"%" )
print('NH:     ',s_input_win[2]," / ",s_flux_win[2],"[TW]"," / ", 
      np.round(s_flux_win[2]/ s_input_win[2]*100),"%" )
print('Equator:',s_input_win[3]," / ",s_flux_win[3],"[TW]"," / ", 
      np.round(s_flux_win[3]/ s_input_win[3]*100),"%" )


# ## How does the ratio of mlb-flux / surface-input depend on the mixed layer depth? (Olbers et al. 2020 propose it should decline with increasing ML)

# In[40]:


plt.figure(figsize=(8,6))
mld_ = mld["win"].values.reshape(6480000); mld_[np.isnan(mld_)] = 0
flux_ = -1*flux_inOlbers["win"].values.reshape(6480000); flux_[np.isnan(flux_)] = 0
input_ = input_inIn["win"].values.reshape(6480000); input_[np.isnan(input_)] = 0

stat_flux = stats.binned_statistic(mld_,flux_,bins=100, statistic="sum")
stat_input = stats.binned_statistic(mld_,input_,bins=100, statistic="sum")

bins = (stat_flux.bin_edges[1:] + stat_flux.bin_edges[:-1])/2
plt.plot(stat_flux.statistic/stat_input.statistic,-bins, label="Winter", lw = lw)


mld_ = mld["sum"].values.reshape(6480000); mld_[np.isnan(mld_)] = 0
flux_ = -1*flux_inOlbers["sum"].values.reshape(6480000); flux_[np.isnan(flux_)] = 0
input_ = input_inIn["sum"].values.reshape(6480000); input_[np.isnan(input_)] = 0

stat_flux = stats.binned_statistic(mld_,flux_,statistic="sum",bins=100)
stat_input = stats.binned_statistic(mld_,input_,statistic="sum", bins=100)

bins = (stat_flux.bin_edges[1:] + stat_flux.bin_edges[:-1])/2
plt.plot(stat_flux.statistic/stat_input.statistic,-bins, label="Summer", lw=lw)
plt.xlim(0,0.5)
plt.ylim(-670,0)
plt.legend(fontsize=fs)

plt.xlabel("Ratio MLB flux /  surface flux", fontsize=fs)
plt.ylabel("Depth of mixed layer [m]",fontsize=fs)
plt.xticks(fontsize=fs)
plt.yticks([-600,-400,-200,0],[600,400,200,0],fontsize=fs)

plt.savefig(figdir+"ratio_vs_mld.png",bbox_inches="tight")


# In[ ]:





# In[ ]:





# In[ ]:




