#!/usr/bin/env python
# coding: utf-8

# In[7]:


import numpy as np
import xarray as xr
import matplotlib.pyplot as plt
import spectools_niw as niw

def sum_regions(dat): # sums northern hemisphere / southern hemisphere and equatorial contributions
    gl = np.sum(dat[:,:] * ga_r01[:,:]).values
    sh = np.sum(dat[0:900,:] * ga_r01[0:900,:]).values
    nh = np.sum(dat[900:,:] * ga_r01[900:,:]).values
    eq = np.sum(dat[800:1000,:] * ga_r01[800:1000,:]).values
    
    return np.round(gl*1e-12,3),np.round(sh*1e-12,3),np.round(nh*1e-12,3),np.round(eq*1e-12,3)

figdir="/home/mpim/m300522/projects/niw_dywin/figs/"


# In[2]:


flux = {}; inpu = {}; # define empty dictionaries for surface input (inpu) and mlb flux (flux)


#read data winter
idir = "/work/mh0256/m300522/dywin/for_paper/feb2021/"


flux["win_wh"] = xr.open_dataset(idir+"isolate_hurricanes_flux+input_r01_inputSW02_fluxOlbSW4.nc").flux_wh # with hurricanes
flux["win_nh"] = xr.open_dataset(idir+"isolate_hurricanes_flux+input_r01_inputSW02_fluxOlbSW4.nc").flux_nh # non hurricanes
inpu["win_wh"] = xr.open_dataset(idir+"isolate_hurricanes_flux+input_r01_inputSW02_fluxOlbSW4.nc").input_wh # with hurricanes
inpu["win_nh"] = xr.open_dataset(idir+"isolate_hurricanes_flux+input_r01_inputSW02_fluxOlbSW4.nc").input_nh # non hurricanes


#read data summer
idir = "/work/mh0256/m300522/dywin/for_paper/sep2020/"
flux["sum"] = xr.open_dataset(idir+"flux_inOlbers_sw4_r01_202008-202009.nc").flux_inIn
inpu["sum"] = xr.open_dataset(idir+"input_inIn_sw02_r01_202008-202009.nc").input_inIn

flux["sum_wh"] = xr.open_dataset(idir+"isolate_hurricanes_flux+input_r01_inputSW02_fluxOlbSW4.nc").flux_wh # with hurricanes
flux["sum_nh"] = xr.open_dataset(idir+"isolate_hurricanes_flux+input_r01_inputSW02_fluxOlbSW4.nc").flux_nh # non hurricanes
inpu["sum_wh"] = xr.open_dataset(idir+"isolate_hurricanes_flux+input_r01_inputSW02_fluxOlbSW4.nc").input_wh # with hurricanes
inpu["sum_nh"] = xr.open_dataset(idir+"isolate_hurricanes_flux+input_r01_inputSW02_fluxOlbSW4.nc").input_nh # non hurricanes

mask_summer = xr.open_dataset(idir+"hurricane_mask_sep2020.nc").mask_input



ga_r01 = xr.open_dataset(idir+"../ga_r01.nc").cell_area


# In[3]:


xx,yy = np.meshgrid(flux["win_nh"].lon,flux["win_nh"].lat)

fig = plt.figure(figsize=(16,10))
high=1e-2

fig.add_subplot(221)
inpu["sum_wh"].plot(vmax=high,vmin=-high,cmap="RdBu_r")

fig.add_subplot(222)
inpu["sum_nh"].plot(vmax=high,vmin=-high,cmap="RdBu_r")

fig.add_subplot(223)
(-1*flux["sum_wh"]).plot(vmax=high,vmin=-high,cmap="RdBu_r")

fig.add_subplot(224)
(-1*flux["sum_nh"]).plot(vmax=high,vmin=-high,cmap="RdBu_r")
#plt.savefig(figdir+"isolate_hurricanes_summer.png",bbox_inches="tight")


# In[4]:


xx,yy = np.meshgrid(flux["win_nh"].lon,flux["win_nh"].lat)

fig = plt.figure(figsize=(16,10))
high=1e-2

fig.add_subplot(221)
inpu["win_wh"].plot(vmax=high,vmin=-high,cmap="RdBu_r")

fig.add_subplot(222)
inpu["win_nh"].plot(vmax=high,vmin=-high,cmap="RdBu_r")

fig.add_subplot(223)
(-1*flux["win_wh"]).plot(vmax=high,vmin=-high,cmap="RdBu_r")

fig.add_subplot(224)
(-1*flux["win_nh"]).plot(vmax=high,vmin=-high,cmap="RdBu_r")

#plt.savefig(figdir+"isolate_hurricanes_winter.png",bbox_inches="tight")


# In[8]:


# Compute integrated rates

s_flux_win_wh = sum_regions(-1*flux["win_wh"])
s_flux_win_nh = sum_regions(-1*flux["win_nh"])
s_flux_sum_wh = sum_regions(-1*flux["sum_wh"])
s_flux_sum_nh = sum_regions(-1*flux["sum_nh"])

s_input_win_wh = sum_regions(inpu["win_wh"])
s_input_win_nh = sum_regions(inpu["win_nh"])
s_input_sum_wh = sum_regions(inpu["sum_wh"])
s_input_sum_nh = sum_regions(inpu["sum_nh"])


# In[9]:




print("SUMMER:   ", "     Input"  ,"            /    ", "MLB Flux   (In brackets Fraction of total)")
print('Hurricanes   : ',s_input_sum_wh[0],"(",np.round(s_input_sum_wh[0]/ (s_input_sum_wh[0]+s_input_sum_nh[0])*100),"% )       ",s_flux_sum_wh[0],"[TW]","(",np.round(s_flux_sum_wh[0]/ (s_flux_sum_wh[0]+s_flux_sum_nh[0])*100),"% )"  )
print('Hurricanes NH: ',s_input_sum_wh[2],"(",np.round(s_input_sum_wh[2]/ (s_input_sum_wh[2]+s_input_sum_nh[2])*100),"% )       ",s_flux_sum_wh[2],"[TW]","(",np.round(s_flux_sum_wh[2]/ (s_flux_sum_wh[2]+s_flux_sum_nh[2])*100),"% )"  )
print('Hurricanes SH: ',s_input_sum_wh[1],"  (",np.round(s_input_sum_wh[1]/ (s_input_sum_wh[1]+s_input_sum_nh[1])*100),"%  )       ",s_flux_sum_wh[1],"[TW]","  (",np.round(s_flux_sum_wh[1]/ (s_flux_sum_wh[1]+s_flux_sum_nh[1])*100),"% )"  )


print()
print("WINTER:   ", "     Input"  ,"            /    ", "MLB Flux   (In brackets Fraction of total)")
print('Hurricanes   : ',s_input_win_wh[0],"(",np.round(s_input_win_wh[0]/ (s_input_win_wh[0]+s_input_win_nh[0])*100),"% )       ",s_flux_win_wh[0],"[TW]","(",np.round(s_flux_win_wh[0]/ (s_flux_win_wh[0]+s_flux_win_nh[0])*100),"% )"  )
print('Hurricanes NH: ',s_input_win_wh[2],"  (",np.round(s_input_win_wh[2]/ (s_input_win_wh[2]+s_input_win_nh[2])*100),"% )        ",s_flux_win_wh[2],"[TW]","  (",np.round(s_flux_win_wh[2]/ (s_flux_win_wh[2]+s_flux_win_nh[2])*100),"% )"  )
print('Hurricanes SH: ',s_input_win_wh[1],"(",np.round(s_input_win_wh[1]/ (s_input_win_wh[1]+s_input_win_nh[1])*100),"% )       ",s_flux_win_wh[1],"[TW]"," (",np.round(s_flux_win_wh[1]/ (s_flux_win_wh[1]+s_flux_win_nh[1])*100),"% )"  )


# ## The above data for hurricane and non-hurricanes can be obtained via the following code

# In[10]:


# separate hurricanes and non-hurricanes via mask
flux = {}; inpu = {}; # define empty dictionaries for surface input (inpu) and mlb flux (flux)


#winter
idir = "/work/mh0256/m300522/dywin/for_paper/feb2021/"
flux["win"] = xr.open_dataset(idir+"flux_inOlbers_sw4_r01_202101-202102.nc").flux_inIn
inpu["win"] = xr.open_dataset(idir+"input_inIn_sw02_r01_202101-202102.nc").input_inIn

mask_winter = xr.open_dataset(idir+"hurricane_mask_feb2021.nc").mask_input


input_wh = inpu["win"].values.copy(); input_wh[mask_winter !=1] = 0.
input_nh = inpu["win"].values.copy(); input_nh[mask_winter ==1] = 0.

flux_wh = flux["win"].values.copy(); flux_wh[mask_winter !=1] = 0.
flux_nh = flux["win"].values.copy(); flux_nh[mask_winter ==1] = 0.


ds = xr.Dataset(
    {
        "flux_wh": (["lat" ,"lon"], flux_wh),
        "flux_nh": (["lat" ,"lon"], flux_nh),

        "input_wh": (["lat" ,"lon"], input_wh),
        "input_nh": (["lat" ,"lon"], input_nh),

    },
    coords={
        "lat": ( flux["win"].lat.values),
        "lon": (flux["win"].lon.values),

    },
)


#ds.to_netcdf(idir+"isolate_hurricanes_flux+input_r01_inputSW02_fluxOlbSW4.nc")


#summer
idir = "/work/mh0256/m300522/dywin/for_paper/sep2020/"
flux["sum"] = xr.open_dataset(idir+"flux_inOlbers_sw4_r01_202008-202009.nc").flux_inIn
inpu["sum"] = xr.open_dataset(idir+"input_inIn_sw02_r01_202008-202009.nc").input_inIn

mask_summer = xr.open_dataset(idir+"hurricane_mask_sep2020.nc").mask_input


input_wh = inpu["sum"].values.copy(); input_wh[mask_winter !=1] = 0.
input_nh = inpu["sum"].values.copy(); input_nh[mask_winter ==1] = 0.

flux_wh = flux["sum"].values.copy(); flux_wh[mask_winter !=1] = 0.
flux_nh = flux["sum"].values.copy(); flux_nh[mask_winter ==1] = 0.

ds = xr.Dataset(
    {
        "flux_wh": (["lat" ,"lon"], flux_wh),
        "flux_nh": (["lat" ,"lon"], flux_nh),

        "input_wh": (["lat" ,"lon"], input_wh),
        "input_nh": (["lat" ,"lon"], input_nh),

    },
    coords={
        "lat": ( flux["sum"].lat.values),
        "lon": (flux["sum"].lon.values),

    },
)


#ds.to_netcdf(idir+"isolate_hurricanes_flux+input_r01_inputSW02_fluxOlbSW4.nc")


# In[ ]:




