#!/usr/bin/env python
# coding: utf-8

# In[2]:


import spectools_niw as niw # the file spectools_niw.py needs to be in the same folder as this script, contains functions
import numpy as np
import xarray as xr
import warnings
warnings.filterwarnings('ignore')
from importlib import reload


# In[3]:


#dirs
sumdir = "/work/mh0256/m300522/dywin/for_paper/sep2020/"
windir = "/work/mh0256/m300522/dywin/for_paper/feb2021/"

#dicts
flux_allf = {}; flux_inIn = {}; flux_inOlbers = {}
input_allf = {}; input_inIn = {}; input_inOlbers = {}


#metas
ga01 = xr.open_dataset(sumdir+"../ga_r01.nc").cell_area

# load data from spectral mlb flux and spectral work at surface
flux_allf["sum"] = xr.open_dataset(sumdir+"flux_allf_202008-202009_r01.nc")["flux_allf_202008-202009"]
flux_allf["win"] = xr.open_dataset(windir+"flux_allf_202101-202102_r01.nc")["flux_allf_202101-202102"]

input_allf["sum"] = xr.open_dataset(sumdir+"input_allf_202008-202009_r01.nc")["input_allf_202008-202009"]
input_allf["win"] = xr.open_dataset(windir+"input_allf_202101-202102_r01.nc")["input_allf_202101-202102"]


# In[ ]:


## compute & save NEAR-INERTIAL SURFACE INPUT (WINTER)
#in respective frequency band around the local f
# dt=1 gives the timestep of the input data, spec_width=0.2 means that we integrate in the range 0.8f-1.2f
#(spec_width=0.3 would mean 0.7f-1.3f)
ds = xr.Dataset(
    {
        "input_inIn": (["lat", "lon"], niw.cut_inertial_input_01(input_allf["win"].values,input_allf["win"].freq.values,
                                                                 dt=1, spec_width=0.2)), # input was computed using 1hourly data
    },
    coords={
        "lon": ( flux_allf["win"].lon),
        "lat": ( flux_allf["win"].lat),
    },
)
ds.to_netcdf(windir+"input_inIn_sw02_r01_202101-202102.nc")


# In[ ]:


## compute & save NEAR-INERTIAL SURFACE INPUT (SUMMER)

input_inIn["sum"] = xr.Dataset(
    {
        "input_inIn": (["lat", "lon"], niw.cut_inertial_input_01(input_allf["sum"].values,input_allf["sum"].freq.values,
                                                                 dt=1,spec_width=0.2)),
    },
    coords={
        "lon": ( flux_allf["win"].lon),
        "lat": ( flux_allf["win"].lat),
    },
)
input_inIn["sum"].to_netcdf(sumdir+"input_inIn_sw02_r01_202008-202009.nc")


# In[ ]:


## compute & save NEAR-INERTIAL MLB FLUX (WINTER)
# dt=3 because the mlb data of w and p are available only 3hourly
# This method selects all frequencies higher than the intertial frequency. This is what we use in the paper

ds = xr.Dataset(
    {
        "flux_inIn": (["lat", "lon"], niw.cut_inertial_radiation_olbers_01(flux_allf["win"].values,flux_allf["win"].freq.values,
                                                                           dt=windt,spec_width=4)),
    },
    coords={
        "lon": ( flux_allf["win"].lon),
        "lat": ( flux_allf["win"].lat),
    },
)
ds.to_netcdf(windir+"flux_inOlbers_sw4_r01_202101-202102.nc")


# In[ ]:


## compute & save NEAR-INERTIAL MLB FLUX (SUMMER)
# This method selects all frequencies higher than the intertial frequency. This is what we use in the paper
ds = xr.Dataset(
    {
        "flux_inIn": (["lat", "lon"], niw.cut_inertial_radiation_olbers_01(flux_allf["sum"].values,flux_allf["sum"].freq.values,
                                                                           dt=sumdt,spec_width=4)),
    },
    coords={
        "lon": ( flux_allf["win"].lon),
        "lat": ( flux_allf["win"].lat),
    },
)
ds.to_netcdf(sumdir+"flux_inOlbers_sw4_r01_202008-202009.nc")


# In[ ]:


## compute & save NEAR-INERTIAL MLB FLUX (WINTER)
# dt=3 because the mlb data of w and p are available only 3hourly
# this method selects a spectral band that is symmetric about the equator as for the surface input, 0.8f-1.2f

flux_inIn["win"] = xr.Dataset(
    {
        "flux_inIn": (["lat", "lon"], niw.cut_inertial_radiation_01(flux_allf["win"].values,flux_allf["win"].freq.values,
                                                                    dt=3, spec_width=0.2)),
    },
    coords={
        "lon": ( flux_allf["win"].lon),
        "lat": ( flux_allf["win"].lat),
    },
)
flux_inIn["win"].to_netcdf(windir+"flux_inIn_sw02_r01_202101-202102.nc")


# In[ ]:


## compute & save NEAR-INERTIAL MLB FLUX (SUMMER)
# this method selects a spectral band that is symmetric about the equator as for the surface input, 0.8f-1.2f
flux_inIn["sum"] = xr.Dataset(
    {
        "flux_inIn": (["lat", "lon"], niw.cut_inertial_radiation_01(flux_allf["sum"].values,flux_allf["sum"].freq.values,
                                                                    dt=1,spec_width=0.2)),
    },
    coords={
        "lon": ( flux_allf["win"].lon),
        "lat": ( flux_allf["win"].lat),
    },
)
flux_inIn["sum"].to_netcdf(windir+"flux_inIn_sw02_r01_202008-202009.nc")


# In[ ]:




