import xarray as xr
import numpy as np
import matplotlib.pyplot as plt
from numba import jit, prange
from scipy import stats
from scipy import signal
import numpy.ma as ma
import scipy.ndimage as ndimage

# add meta information for grid 0.1 deg
idir01 = "/work/mh0256/m300522/DyWin3/second-phase-40days/deep01/"
ds01 = xr.open_dataset(idir01+"flux_allf_202101-202102_r01.nc")
ga01 = xr.open_dataset(idir01+"ga01.nc").cell_area
omega = 1/24 # unit 1/h
cori01 = (2*omega*np.sin(np.deg2rad(np.array(ds01.lat))))  # units 1/h

# add meta information for grid 0.4 deg
idir04 = "/work/mh0256/m300522/DyWin3/second-phase-40days/deep/"
ga04 = xr.open_dataset(idir04+"ga04.nc").cell_area
omega = 1/24 # unit 1/h
cori04 = (2*omega*np.sin(np.deg2rad(np.array(ga04.lat))))  # units 1/h

def get_cori(lat):
    omega = 1/24 # unit 1/h
    cori = (2*omega*np.sin(np.deg2rad(np.array(lat))))
    ds = xr.Dataset(
    {
        "cori": (["lat"], cori),
    },
    coords={

        "lat": ( np.array(lat)),

    },
        )
    return ds.cori

def get_cori_single(lat):
    omega = 1/24 # unit 1/h
    cori = (2*omega*np.sin(np.deg2rad(np.array(lat))))
    return cori

def get_meta(reso):
    if reso=="01":
        idir01 = "/work/mh0256/m300522/DyWin3/second-phase-40days/deep01/"
        ds01 = xr.open_dataset(idir01+"flux_allf_202101-202102_r01.nc")
        ga = xr.open_dataset(idir01+"ga01.nc").cell_area
        omega = 1/24 # unit 1/h
        cori = (2*omega*np.sin(np.deg2rad(np.array(ds01.lat))))
        lats = ds01.lat
    elif reso=="04":
        # add meta information for grid 0.4 deg
        idir04 = "/work/mh0256/m300522/DyWin3/second-phase-40days/deep/"
        ga = xr.open_dataset(idir04+"ga04.nc").cell_area
        omega = 1/24 # unit 1/h
        cori = (2*omega*np.sin(np.deg2rad(np.array(ga04.lat))))  # units 1/h
        lats = ga.lat
    else:
        print("resolution not available")
    return cori,ga,lats
    

def cut_inertial_radiation_01(B,f,dt,spec_width=0.2):
    inertial_input = np.zeros((B.shape[1],B.shape[2]))
    for jj in range(B.shape[1]):
        corio = get_meta(reso="01")[0][jj].copy()
        if (jj%50 == 0):
            print(jj)
        finput = B[:,jj,:].copy()

        if corio < 0:
            mask1 = f<-corio*(1-spec_width)
            mask2 = f>-corio*(1+spec_width)
            finput[mask1+mask2,:] = 0

        if corio >= 0:
            mask1 = f<-corio*(1+spec_width)
            mask2 = f>-corio*(1-spec_width)
            finput[mask1+mask2,:] = 0

        inertial_input[jj,:] = np.sum(finput*(f[2]-f[1])*dt,axis=0) *2
        # needs to be multiplied because timeseries is real and therefore the inertial peak appears 
        # symetrically at f and -f
        inertial_input[inertial_input==0] = np.nan 
    return inertial_input

def cut_inertial_radiation_olbers_01(B,f,dt,spec_width=1):
    inertial_input = np.zeros((B.shape[1],B.shape[2]))
    
    for jj in range(B.shape[1]):
        corio = get_meta(reso="01")[0][jj].copy()
        if (jj%50 == 0):
            print(jj)
        finput = B[:,jj,:].copy()

        if corio < 0:
            mask1 = f<-corio*0.95
            mask2 = f>-corio*(1+spec_width)
            finput[mask1+mask2,:] = 0

        if corio >= 0:
            mask1 = f<-corio*(1+spec_width)
            mask2 = f>-corio*0.95
            finput[mask1+mask2,:] = 0

        inertial_input[jj,:] = np.sum(finput*(f[2]-f[1])*dt,axis=0) * 2
        # needs to be multiplied because timeseries is real and therefore the inertial peak appears 
        # symetrically at f and -f
        inertial_input[inertial_input==0] = np.nan 
    return inertial_input


def cut_inertial_input_01(B,f,dt,spec_width=0.2):
    inertial_input = np.zeros((B.shape[1],B.shape[2]))
    
    for jj in prange(B.shape[1]):
        corio = get_meta(reso="01")[0][jj].copy()
        if (jj%100 == 0):
            print(jj)


        finput = B[:,jj,:].copy()
        #finput[finput==0] = np.nan

        if corio < 0:
            mask1 = f<-corio*(1-spec_width)
            mask2 = f>-corio*(1+spec_width)
            finput[mask1+mask2,:] = 0

        if corio >= 0:
            mask1 = f<-corio*(1+spec_width)
            mask2 = f>-corio*(1-spec_width)
            finput[mask1+mask2,:] = 0

        inertial_input[jj,:] = np.sum(finput*(f[2]-f[1])*dt,axis=0)
        inertial_input[inertial_input==0] = np.nan 
    return inertial_input


def cut_inertial_radiation_04(B,f,dt,spec_width=0.2): # input complex freq
    inertial_input = np.zeros((B.shape[1],B.shape[2]))
    for jj in range(B.shape[1]):
        corio = cori04[jj].copy()
        if (jj%50 == 0):
            print(jj)
        finput = B[:,jj,:].copy()

        if corio < 0:
            mask1 = f<-corio*(1-spec_width)
            mask2 = f>-corio*(1+spec_width)
            finput[mask1+mask2,:] = 0

        if corio >= 0:
            mask1 = f<-corio*(1+spec_width)
            mask2 = f>-corio*(1-spec_width)
            finput[mask1+mask2,:] = 0

        inertial_input[jj,:] = np.sum(finput*(f[2]-f[1])*dt,axis=0) *2
        # needs to be multiplied because timeseries is real and therefore the inertial peak appears 
        # symetrically at f and -f
        inertial_input[inertial_input==0] = np.nan 
    return inertial_input

def cut_inertial_radiation_allres(B,f,dt,spec_width=0.2): # input complex freq
    inertial_input = np.zeros((B.shape[1],B.shape[2]))
    lats = np.arange(-90,90,180/B.shape[1])
    for jj in range(B.shape[1]):
        corio = get_cori(lats)[jj].values
        if (jj%50 == 0):
            print(jj)
        finput = B[:,jj,:].copy()

        if corio < 0:
            mask1 = f<-corio*(1-spec_width)
            mask2 = f>-corio*(1+spec_width)
            finput[mask1+mask2,:] = 0

        if corio >= 0:
            mask1 = f<-corio*(1+spec_width)
            mask2 = f>-corio*(1-spec_width)
            finput[mask1+mask2,:] = 0

        inertial_input[jj,:] = np.sum(finput*(f[2]-f[1])*dt,axis=0) *2
        # needs to be multiplied because timeseries is real and therefore the inertial peak appears 
        # symetrically at f and -f
        inertial_input[inertial_input==0] = np.nan 
    return inertial_input

def cut_inertial_surface_input_allres(B,f,dt,spec_width=0.2): # input complex freq
    inertial_input = np.zeros((B.shape[1],B.shape[2]))
    lats = np.arange(-90,90,180/B.shape[1])
    for jj in range(B.shape[1]):
        corio = get_cori(lats)[jj].values
        if (jj%200 == 0):
            print(jj)
        finput = B[:,jj,:].copy()

        if corio < 0:
            mask1 = f<-corio*(1-spec_width)
            mask2 = f>-corio*(1+spec_width)
            finput[mask1+mask2,:] = 0

        if corio >= 0:
            mask1 = f<-corio*(1+spec_width)
            mask2 = f>-corio*(1-spec_width)
            finput[mask1+mask2,:] = 0

        inertial_input[jj,:] = np.sum(finput*(f[2]-f[1])*dt,axis=0)
        inertial_input[inertial_input==0] = np.nan 
    return inertial_input

def cut_inertial_radiation_olbers_04(B,f,dt,spec_width=1):
    inertial_input = np.zeros((B.shape[1],B.shape[2]))
    
    for jj in range(B.shape[1]):
        corio = cori04[jj].copy()
        if (jj%50 == 0):
            print(jj)
        finput = B[:,jj,:].copy()

        if corio < 0:
            mask1 = f<-corio*0.95
            mask2 = f>-corio*(1+spec_width)
            finput[mask1+mask2,:] = 0

        if corio >= 0:
            mask1 = f<-corio*(1+spec_width)
            mask2 = f>-corio*0.95
            finput[mask1+mask2,:] = 0

        inertial_input[jj,:] = np.sum(finput*(f[2]-f[1])*dt,axis=0) * 2
        # needs to be multiplied because timeseries is real and therefore the inertial peak appears 
        # symetrically at f and -f

        inertial_input[inertial_input==0] = np.nan 
    return inertial_input

@jit(nopython=False, parallel=True)
def spectral_flux(w,p,f,dt):
    B = np.zeros((p.shape[0],p.shape[1],p.shape[2]))
    for jj in range(p.shape[1]):
        if (jj%50 == 0):
            print(jj)
        w_ = w[:,jj,:]
        w_[np.isnan(w_)] = 0.
        w_ = signal.detrend(w_,axis=0,type="linear")
        p_ = p[:,jj,:]
        p_[np.isnan(p_)] = 0.
        p_ = signal.detrend(p_,axis=0,type="linear")
        fw = np.fft.fftshift(np.fft.fft(w_,axis=0),axes=0)
        fg = np.fft.fftshift(np.fft.fft(p_,axis=0),axes=0)

        finput = np.real(fw*fg.conj())*(f[2]-f[1])*dt
        
        B[:,jj,:] = finput
    return B

@jit(nopython=False, parallel=True)
def spectral_density(p,f,dt):
    B = np.zeros((p.shape[0],p.shape[1],p.shape[2]))
    for jj in range(p.shape[1]):
        if (jj%200 == 0):
            print(jj)
        p_ = p[:,jj,:]
        p_[np.isnan(p_)] = 0.
        p_ = signal.detrend(p_,axis=0,type="linear")
        fp = np.fft.fftshift(np.fft.fft(p_,axis=0),axes=0)

        finput = np.real(fp*fp.conj())*(f[2]-f[1])*dt

        B[:,jj,:] = finput
    return B

@jit(nopython=False, parallel=True)
def filter_niw_04(p,f,olbers=True): # very important to take real frequency as input (np.fft.rfftfreq(...))
    fp = np.fft.rfft(p,axis=0)
    if olbers==True:
        fp_cut = cut_niw_olbers_04(fp,f)
    p_cut = np.fft.irfft(fp_cut,axis=0)
    return p_cut

def cut_niw_olbers_04(B,f,spec_width=1): # go up to 2f
    inertial_input = np.zeros((B.shape[0],B.shape[1],B.shape[2]),dtype="complex")
    for jj in range(B.shape[1]):
        corio = get_meta(reso="04")[0][jj].copy()
        if (jj%50 == 0):
            print(jj)
        finput = B[:,jj,:].copy()

        if corio < 0:
            mask1 = f<-corio*0.95
            mask2 = f>-corio*(1+spec_width) 
            finput[mask1+mask2,:] = 0
        
        if corio >= 0:
            mask1 = f<corio*0.95#*(1-spec_width)
            mask2 = f>corio*(1+spec_width)
            finput[mask1+mask2,:] = 0
        
        inertial_input[:,jj,:]  = finput.copy()
    return inertial_input

@jit(nopython=False, parallel=True)
def filter_niw_01(p,f,spec_width=4,olbers=True): # very important to take real frequency as input (np.fft.rfftfreq(...))
    fp = np.fft.rfft(p,axis=0)
    if olbers==True:
        fp_cut = cut_niw_olbers_01(fp,f,spec_width=spec_width)
    else:
        fp_cut = cut_niw_01(fp,f,spec_width=spec_width) 

    p_cut = np.fft.irfft(fp_cut,axis=0)
    return p_cut

def cut_niw_olbers_01(B,f,spec_width=1): # go up to 2f
    inertial_input = np.zeros((B.shape[0],B.shape[1],B.shape[2]),dtype="complex")
    for jj in range(B.shape[1]):
        corio = get_meta(reso="01")[0][jj].copy()
        if (jj%200 == 0):
            print(jj)
        finput = B[:,jj,:].copy()

        if corio < 0:
            mask1 = f<-corio*0.95
            mask2 = f>-corio*(1+spec_width)
            finput[mask1+mask2,:] = 0

        if corio >= 0:
            mask1 = f<corio*0.95#*(1-spec_width)
            mask2 = f>corio*(1+spec_width)
            finput[mask1+mask2,:] = 0

        inertial_input[:,jj,:]  = finput.copy()
    return inertial_input

def cut_niw_01(B,f,spec_width=0.2): # go up to 2f
    inertial_input = np.zeros((B.shape[0],B.shape[1],B.shape[2]),dtype="complex")
    for jj in range(B.shape[1]):
        corio = get_meta(reso="01")[0][jj].copy()
        if (jj%200 == 0):
            print(jj)
        finput = B[:,jj,:].copy()
        if corio < 0:
            mask1 = f<-corio*(1-spec_width)
            mask2 = f>-corio*(1+spec_width)
            finput[mask1+mask2,:] = 0

        if corio >= 0:
            mask1 = f<corio*(1-spec_width)
            mask2 = f>corio*(1+spec_width)
            finput[mask1+mask2,:] = 0

        inertial_input[:,jj,:]  = finput.copy()
    return inertial_input

def smooth2d(data,sigma):
    #sigma=1
    #a = (data.mask == True)
    #data[a] = 0
    datag = ma.empty(data.shape)
    #datag = ma.empty(data.shape)
    datag = ndimage.gaussian_filter(data, sigma, order=0)
    #atag.mask[a] = True
    return datag

from numba import jit, prange
@jit(nopython=False, parallel=True)
def spectral_surface_input(taux,tauy,u,v,f,dt):
    B = np.zeros((taux.shape[0],taux.shape[1],taux.shape[2]))
    for jj in range(taux.shape[1]):
#        if (jj%50 == 0):
 #           print(jj)

        wind = taux[:,jj,:] + tauy[:,jj,:]*1j
        flow = u[:,jj,:] + v[:,jj,:]*1j

        fw = np.fft.fftshift(np.fft.fft(wind,axis=0),axes=0)
        fg = np.fft.fftshift(np.fft.fft(flow,axis=0),axes=0)

        finput = np.real(fw*fg.conj())*(f[2]-f[1])*dt
        
        B[:,jj,:] = finput
    return B


